# Provides the Tk gui routines.

package provide tkgui 0.1

namespace eval chat::gui {
    set update 10
}

# chat::gui::dodisplay --
#
#	Take care of keeping the chat lines up to date.
#
# Arguments:
#	textwidget - widget to update.
#	nick - nick of the user.

proc chat::gui::dodisplay {textwidget nick} {
    $textwidget delete 0.0 end
    foreach line [chat::model::getlines $nick] {
	$textwidget insert end "$line\n"
    }
    after [expr {10 * 1000}] [list chat::gui::dodisplay $textwidget $nick]
}

# chat::gui::dowho --
#
#	Keep 'who' information up to date.
#
# Arguments:
#	widget - widget to update.
#	nick - nick of the user.

proc chat::gui::dowho {widget nick} {
    variable update
    $widget delete 0 end
    foreach dt [chat::model::who $nick] {
	$widget insert end $dt
    }
    after [expr {$update * 1000}] [list chat::gui::dowho $widget $nick]
}

# chat::gui::main --
#
#	Main screen.
#
# Arguments:
#	nick - nick of the user.
#
# Side Effects:
#	Sets up main display.
#	Sets up dodisplay and dowho routines.
#
# Results:
#	None.

proc chat::gui::main {nick} {
    label .title -text "Tcl Chat: $nick" -background "#009900"
    label .online -text "Currently on line:"
    grid .title -sticky we -row 0 -column 0 -columnspan 2

    # Display area
    text .display
    grid .display -sticky nsew -row 1 -column 0 -rowspan 2
    chat::gui::dodisplay .display $nick

    # Users on-line
    listbox .users
    grid .online -row 1 -column 1
    grid .users -sticky nsew -row 2 -column 1
    grid columnconfigure . 0 -weight 2
    grid rowconfigure . 2 -weight 4
    chat::gui::dowho .users $nick

    # Input frame
    frame .input
    entry .input.msg

    set cmd {
	chat::logic::chat $nick [.input.msg get]
	.input.msg delete 0 end
    }
    set cmd [subst -nocommands $cmd]

    bind .input.msg <Return> $cmd
    button .input.send -text "Send" -command $cmd

    grid .input -sticky wes -columnspan 2
    grid .input.msg -sticky we -row 0 -column 0
    grid columnconfigure .input 0 -weight 1
    grid columnconfigure .input 1 -weight 0
    grid .input.send -sticky e -row 0 -column 1
}

# chat::gui::updatemain --
#
#	Called to update the main screen.
#
# Arguments:
#	nick - nick of the user.
#
# Side Effects:
#	Updates the main screen.
#
# Results:
#	None.

proc chat::gui::updatemain {nick} {
    chat::gui::dowho .users $nick
    chat::gui::dodisplay .display $nick
}

# chat::gui::error --
#
#	Error display.
#
# Arguments:
#	txt - error message to display.

proc chat::gui::error {txt} {
    ::error $txt
}

proc chat::gui::newuser {nick pass} {
    frame .newuser
    label .newuser.title -text "New Tcl Chat User: $nick" -background "#009900"
    label .newuser.txt -text "All fields are optional, except for the password confirmation"


    label .newuser.namelabel -text "Name:"
    entry .newuser.name

    label .newuser.urllabel -text "Url:"
    entry .newuser.url

    label .newuser.emaillabel -text "Email:"
    entry .newuser.email

    label .newuser.pwconfirmlabel -text "Confirm Password"
    entry .newuser.pwconfirm -show *

    set cmd {
	chat::logic::newuser $nick $pass [.newuser.pwconfirm get]
	destroy .newuser
    }
    set cmd [subst -nocommands $cmd]

    button .newuser.createuser -text "Create User" -command $cmd

    grid .newuser.title -sticky we -row 0 -column 0 -columnspan 2
    grid .newuser.txt -sticky ew -columnspan 2
    grid .newuser.namelabel .newuser.name -sticky ew
    grid .newuser.urllabel .newuser.url -sticky ew
    grid .newuser.emaillabel .newuser.email -sticky ew
    grid .newuser.pwconfirmlabel .newuser.pwconfirm -sticky ew
    grid .newuser.createuser -sticky ew -columnspan 2
    grid .newuser
}

proc chat::gui::login {} {
    frame .login

    label .login.title -text "Tcl Chat Login" -background "#009900"

    label .login.nicklabel -text "Nick:"
    entry .login.nick
    button .login.login -text "Login" -command {
	chat::logic::login [.login.nick get] [.login.pass get]
	destroy .login
    }

    label .login.passlabel -text "Password:"
    entry .login.pass -show *
    button .login.createnew -text "Create New User" -command {
	chat::gui::newuser [.login.nick get] [.login.pass get]
	destroy .login
    }

    grid .login.title -columnspan 3 -sticky ew
    grid .login.nicklabel .login.nick .login.login -sticky we
    grid .login.passlabel .login.pass .login.createnew -sticky we
    grid .login
}

proc chat::gui::quit {} {
    exit
}